#import "HierSocialEntropy.h"

#import "UDiamCluster.h"
#import <misc.h>


@implementation HierSocialEntropy
  
static int 
compare_doubles (const double *a, const double *b)
{
  return *a < *b ? -1: *a !=*b ;
}

static id myZone;

+(double) calculate: aZone N: (int) num DissMatr: (double * *) diss
{
  int i,j;
  double entropyForK;
  int thresholdCoord = 0;
  id  aClusterTool = nil;
  id clusters;
  double *  thresholdArray;
  double HSE;

  myZone = aZone;

#ifdef debug
  fprintf(stderr,"distances are \n");
  for (i = 0; i < num; i++)
    {
      for (j = 0; j < num; j++)
      {
	fprintf (stderr, " diss[%d][%d]=%f",i,j,diss[i][j]);
      }
      fprintf( stderr, "\n");
    }
  fprintf( stderr, "\n");
#endif

  thresholdArray = [myZone alloc: (1+(num * (num-1 / 2.0)))*sizeof(double)];

  thresholdArray[0] = 0.0;

  for (i = 0; i < num; i++)
    {
    for (j = i+1; j < num; j++)
      {
	thresholdCoord++;
	thresholdArray [thresholdCoord] = diss [i][j];
      }
    }

#ifdef DEBUG
  fprintf(stderr,"before sorting, the threshold values are: \n" );
  for (i = 0; i< thresholdCoord+1; i++)
    fprintf(stderr," %f " , thresholdArray[i]);
  fprintf(stderr,"\n");
#endif

  qsort(thresholdArray,thresholdCoord+1,sizeof(double),( int(*)( const void*,const void*))compare_doubles);

#ifdef DEBUG
  fprintf (stderr, "\n num is %d and num of threshold coordinates is %d \n", num, thresholdCoord+1);
  fprintf(stderr,"thresholds are:" );
  for (i = 0; i< thresholdCoord+1; i++)
    fprintf(stderr," %f " , thresholdArray[i]);
  fprintf(stderr,"\n");
#endif

  aClusterTool = [UDiamCluster create: myZone N: num DissMatr: diss];
  HSE=0;
  for (j=0; j < thresholdCoord+1 ; j++)
    {
      double width = thresholdArray[j+1] - thresholdArray[j]; 
      clusters = [aClusterTool  clusterK: thresholdArray[j]]; 
      entropyForK = [self calculateEntropyForClusters: clusters];
      HSE += (entropyForK * width);
#ifdef DEBUG
      [aClusterTool printMembers];
      fprintf(stderr, "K: %f entropyForK: %f, HSE = %f \n", .2+thresholdArray[j],entropyForK, HSE);
#endif
     }

  [myZone free: thresholdArray];
  [aClusterTool drop];
  return HSE;
}



+ (double) calculateEntropyForClusters: (id <List>) list
{
  int i = 0;
  int j = 0;
  int numClusters = 0;
  int totalMemberships = 0;
  int * sizes;
  double entropy = 0;

  id index;
  AVLSet * aCluster;

  numClusters = [list getCount];

  sizes=[ myZone  allocBlock: numClusters * sizeof (int) ];
  
  index = [ list begin: myZone];
  for ( aCluster = [index next]; [index getLoc] == Member; aCluster = [index next])
  {
    sizes[i] = [aCluster getCount];
    totalMemberships += sizes[i];
    i++;
  }

  for ( j = 0; j < numClusters; j++) 
  {
    double  p;
        
    p = (double)sizes[j] / (double)totalMemberships;
    entropy += p *(-1.0)* log2(p);
  }

   [index drop];
   return entropy;
}

@end








