#import "UDiamCluster.h"

#import <objectbase/SwarmObject.h>

#include <misc/avl.h> //for the AVLtree
#import <math.h>
#import <misc.h>


//A small wrapper class to hold integers so we can insert them into
//swarm lists

@interface Integer: SwarmObject
{
  int value;
  member_t link;
}
- setValue: (int)value;
- (int)getValue;
@end


@implementation Integer
- setValue: (int)theValue
{
  value = theValue;
  return self;
}
- (int)getValue
{
  return value;
}
@end



@implementation AVLSet

//compare functions for AVLtrees must take 3 args, 2 objects and 1 optional parameter.
static int
compare_agent_ids (const void *A, const void *B, void *PARAM)
{
  int a = (unsigned) A;
  int b = (unsigned) B;

  if (a  < b )
    return -1;

  return a > b;
}

+createBegin: aZone  
{
  AVLSet * obj; 
  obj = [super createBegin: aZone];
  obj->myTree = avl_create ( compare_agent_ids, NULL);
  return obj;
}

-createEnd
{
  return [super createEnd];
}

+createBegin: (int (*)(const void*,const void*,void*)) comparFunc  Param: (void *) param Zone: aZone
{
  AVLSet * obj; 
  obj = [super createBegin: aZone];
  obj->myTree = avl_create (comparFunc, param);
  return obj;
}

//add an object if it is not currently in the set
//if anObject is in the set, then return that value.
//does nothing and returns NO otherwise
-(BOOL) add: anObject
  {
    id value = avl_insert (myTree, anObject );//uses integer keys;
    if (value == NULL)
      return YES;
    else return NO;
  }

// if anObject matching key exists, replace it, return old one.
// otherwise do nothing and return nil
- (id *) replace: anObject
{
  void * value;
  if (( value = avl_find(myTree, anObject) ))
    {
      avl_replace (myTree, anObject);
    }
  return value; //have to worry about NULL being different from nil?
}


// Super power insert. Inserts if object does not exist, replaces if object does.
// returns object being replaced if there is one.
- forcefulReplace: anObject
  {
    id value = avl_replace (myTree, anObject );//uses integer keys;
    if (value != NULL) 
      return value;
    else return nil;
  }


-(void)  walk: (void (*)(void*,void*)) aFunction Param: (void *) PARAM
{
  avl_walk(myTree , aFunction, PARAM);
}


- traverse: (avl_traverser *) travPtr
{
  id anObject;

  anObject = avl_traverse (myTree, travPtr) ;
  
 return anObject;
}

//to implement the KeyedCollection Protocol:

- at: aKey
{
  void * value=avl_find(myTree, aKey);
   return value;
}


- removeKey: aKey
{
  return avl_delete(myTree, aKey);
}


- (BOOL) containsKey: aKey
{
  if (avl_find(myTree, aKey))
    return YES;
  else
    return NO;
}

- createIndex: (id <Zone>)aZone fromMember: anObject 
{
  raiseEvent(WarningMessage,"Not Implemented");
  return self;
}

//For Collections protocol
- (int) getCount
{
  return avl_count(myTree);
}

- (void) drop
{
  avl_destroy(myTree,NULL); //does not drop objects tree refers to
  [super drop];
}

- removeAll
{
  avl_destroy(myTree,NULL);
  return self;
}


-  deleteAll
{
  avl_free(myTree);  //drops objects in the tree and tree too
  return self;
}


@end




@implementation UDiamCluster
  
static int
compare_agent_position (const void *A, const void *B, void *PARAM)
{
  int a =  [(const id) A getValue];
  int b =  [(const id) B getValue];
  if (a  < b )
    return -1;
  return a> b;
}


#define INTEGER(value) \
  [[[Integer createBegin: [self getZone]] setValue: value] createEnd]


- clusterK: (double) threshold;
{
  [self agglomerateClusters: clusterList Threshold: threshold];
  [self elimRedundantClusters: clusterList];
  return clusterList;
}


+ create: aZone N: (int) num DissMatr: (double **) diss
{
  UDiamCluster * obj;

  obj = [UDiamCluster createBegin: aZone];
  obj -> N = num;
  obj -> dissMatrix = diss;
  obj = [obj createEnd];

  [obj init];
  return obj;
}

-init
{
  int i = 0; 
  Integer * intValue = nil ;
  clusterList = [List create: [self getZone]];
  population = [ [self getZone] allocBlock: N*sizeof(Integer *)];
  for( i = 0; i < N; i++)
    {
      AVLSet* aCluster = nil;
      intValue = INTEGER(i);
      population[i] = intValue ;
      aCluster = [[AVLSet createBegin: compare_agent_position Param: NULL Zone: [self getZone] ] createEnd] ;
      [aCluster add: population[i]];     
      [clusterList addLast: aCluster ];
    }
  return self;
}

- agglomerateClusters: list Threshold: (double) k
{
  int i;
  id index = nil;
  int toofar = 0;
  AVLSet *  aCluster;

static void
check_dist (void * a, void * PARAM)
{
  int A = [(Integer *) a getValue];
  int B = [(Integer *) PARAM getValue];

  if (A != B)
    {
      double dist= dissMatrix[A][B];
      if ( dist > k  ) toofar=1;
    }
  else
    toofar=1;
}

  for ( i = 0; i < N; i ++)
    {
      index= [list begin: [self getZone] ];
      for (aCluster= [index next]; [index getLoc] ==Member; aCluster=[index next])
	{ 
	  toofar=0;
 	  [aCluster walk: (void (*)(void*,void*)) check_dist Param: population[i] ];  
	  if (toofar == 0) [aCluster add: population[i] ];  
	}
      [index drop];
    }

#ifdef DEBUG
  index= [list begin: [self getZone] ];
  for (aCluster= [index next]; [index getLoc] ==Member; aCluster=[index next])
    { 
      int count = [aCluster getCount];
      fprintf(stderr,"After agglom,mem is %d \n", count);
    }
  [index drop];
#endif
  
  return self;

}


static void
print_int (void * a, void * PARAM)
{
  int A = [(Integer *) a getValue];
  fprintf(stderr," %d",A); 
}

- elimRedundantClusters: (id <List>) list
{
  id index1= [list begin: [self getZone]];
  id reaperList = [List create: [self getZone]];

  AVLSet * clust1 = nil;
  AVLSet * clust2 = nil;

  for(clust1=[index1 next]; [index1 getLoc] == Member; clust1=[index1 next])
    {
      id node1, node2;
      int count;
      id index2= [list begin: [self getZone]];
      [index2 setOffset : [index1 getOffset]];
      
      for ( clust2 = [index2 next]; [index2 getLoc] == Member; clust2 = [index2 next])
	{
	  int same = 0; 
	  if ( (count = [clust1 getCount]) == [clust2  getCount] )
	    {
	      avl_traverser trav1 = {0};
	      avl_traverser trav2 = {0};
	 	   
	      while (( node1 = [clust1 traverse:  (avl_traverser *) &trav1]) && (node2 = [clust2 traverse:  (avl_traverser *) &trav2] ))
		{
		  if ( [node1 getValue] == [node2 getValue] )
		    {
		      same++;
		    }
		}
	    }
	  
	  if ( same == count ) 
	    {
	      AVLSet * catch = [index1 remove];  //remove that nil from index1,
	      [reaperList addLast: catch];
	      break;
	    }
	}
      [index2 drop];
    }
  [index1 drop];

#ifdef DEBUG 
  [self printMembers];
#endif

  return self;
}

- printMembers
{
  id index = nil;
  AVLSet * aCluster;
  index= [clusterList begin: [self getZone] ];

  for (aCluster= [index next]; [index getLoc] ==Member; aCluster=[index next])
   { 
     int count = [aCluster getCount];
     fprintf(stderr,"After elim, mem is %d, Members:", count);
     [aCluster walk: print_int Param: NULL];
     fprintf(stderr," \n");
   }    
  [index drop];
  return self;
}


- (void) drop
{
  AVLSet * aCluster;
  id index;
  index = [clusterList begin: [self getZone]];
  for ( aCluster = [index next]; [index getLoc]==Member; aCluster = [index next] )
    [aCluster drop];
  [index drop];
  [clusterList drop];
  [[self getZone] freeBlock: population  blockSize: N*sizeof(Integer *)]; 
  [super drop];
}

@end











